#!/usr/bin/env python

from imiptools.data import get_window_end, Object
from imiptools.dates import format_datetime, get_default_timezone
from imiptools.profile import Preferences
from imip_store import FileStore, FilePublisher
import sys

try:
    user = sys.argv[1]
    store_and_publish = "-s" in sys.argv[2:]
except IndexError:
    print >>sys.stderr, "Need a user, along with the -s option if updating the store."
    sys.exit(1)

preferences = Preferences(user)
tzid = preferences.get("TZID") or get_default_timezone()

# Get the size of the free/busy window.

try:
    window_size = int(preferences.get("window_size"))
except (TypeError, ValueError):
    window_size = 100
window_end = get_window_end(tzid, window_size)

store = FileStore()
publisher = FilePublisher()

# Get all identifiers for events.

uids = store.get_events(user)

all_events = set()
for uid in uids:
    all_events.add((uid, None))
    all_events.update([(uid, recurrenceid) for recurrenceid in store.get_recurrences(user, uid)])

# Filter out cancelled events.

cancelled = store.get_cancellations(user) or []
all_events.difference_update(cancelled)

# Obtain event objects.

objs = []
for uid, recurrenceid in all_events:
    print >>sys.stderr, uid, recurrenceid
    event = store.get_event(user, uid, recurrenceid)
    if event:
        objs.append(Object(event))

# Build a free/busy collection for the given user.

fb = []
for obj in objs:
    participants = {}
    participants.update(obj.get_value_map("ATTENDEE"))
    participants.update(obj.get_value_map("ORGANIZER"))

    for participant, participant_attr in participants.items():

        # Only consider events where this user actually participates.

        if participant == user:
            if participant_attr.get("PARTSTAT", "NEEDS-ACTION") not in ("DECLINED", "DELEGATED", "NEEDS-ACTION"):

                # Obtain the actual periods associated with the event.

                for start, end in obj.get_periods_for_freebusy(tzid, window_end):
                    fb.append((start, end,
                           obj.get_value("UID"),
                           obj.get_value("TRANSP") or "OPAQUE",
                           format_datetime(obj.get_utc_datetime("RECURRENCE-ID")) or "",
                          ))
            break

fb.sort()

# Store and publish the free/busy collection.

if store_and_publish:
    store.set_freebusy(user, fb)
    publisher.set_freebusy(user, fb)
else:
    for item in fb:
        print "\t".join(item)

# vim: tabstop=4 expandtab shiftwidth=4
